/*============================================================================
 * This function is called at the end of each time step, and has a very
 *  general purpose
 *  (i.e. anything that does not have another dedicated user subroutine)
 *============================================================================*/

/* VERS */

/*
  This file is part of Code_Saturne, a general-purpose CFD tool.

  Copyright (C) 1998-2016 EDF S.A.

  This program is free software; you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free Software
  Foundation; either version 2 of the License, or (at your option) any later
  version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
  details.

  You should have received a copy of the GNU General Public License along with
  this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
  Street, Fifth Floor, Boston, MA 02110-1301, USA.
*/

/*----------------------------------------------------------------------------*/

#include "cs_defs.h"

/*----------------------------------------------------------------------------
 * Standard C library headers
 *----------------------------------------------------------------------------*/

#include <assert.h>
#include <math.h>

#if defined(HAVE_MPI)
#include <mpi.h>
#endif

/*----------------------------------------------------------------------------
 * PLE library headers
 *----------------------------------------------------------------------------*/

#include <ple_coupling.h>

/*----------------------------------------------------------------------------
 *  Local headers
 *----------------------------------------------------------------------------*/

#include "bft_mem.h"
#include "bft_error.h"
#include "bft_printf.h"

#include "fvm_writer.h"

#include "cs_base.h"
#include "cs_field.h"
#include "cs_field_pointer.h"
#include "cs_field_operator.h"
#include "cs_mesh.h"
#include "cs_mesh_quantities.h"
#include "cs_halo.h"
#include "cs_halo_perio.h"
#include "cs_log.h"
#include "cs_parameters.h"
#include "cs_parall.h"
#include "cs_physical_constants.h"
#include "cs_prototypes.h"
#include "cs_rotation.h"
#include "cs_time_moment.h"
#include "cs_time_step.h"
#include "cs_turbomachinery.h"
#include "cs_selector.h"

#include "cs_post.h"

/*----------------------------------------------------------------------------
 *  Header for the current file
 *----------------------------------------------------------------------------*/

#include "cs_prototypes.h"
#include "1d_solution_functions.h"

/*----------------------------------------------------------------------------*/

BEGIN_C_DECLS

/*----------------------------------------------------------------------------*/
/*!
 * \file cs_user_extra_operations.c
 *
 * \brief This function is called at the end of each time step, and has a very
 * general purpose (i.e. anything that does not have another dedicated
 * user subroutine)
 *
 */
/*----------------------------------------------------------------------------*/

/*============================================================================
 * User function definitions
 *============================================================================*/

/*----------------------------------------------------------------------------*/
/*!
 * \brief This function is called at the end of each time step.
 *
 * It has a very general purpose, although it is recommended to handle
 * mainly postprocessing or data-extraction type operations.
 */
/*----------------------------------------------------------------------------*/

void
cs_user_extra_operations(cs_domain_t     *domain)
{
  FILE *file = NULL;
  const cs_lnum_t n_cells     = cs_glob_mesh->n_cells;
  const cs_lnum_t n_cells_ext = cs_glob_mesh->n_cells_with_ghosts;
  const cs_real_3_t *cell_cen =
    (const cs_real_3_t *)cs_glob_mesh_quantities->cell_cen;
  const int n_fields = cs_field_n_fields();
  cs_time_step_t *ts = cs_get_glob_time_step();

  int irangv, nitmx;
  cs_lnum_t iel, nptx;
  cs_solving_info_t sinfo;

  /* convergence test */
  bool cved = true;

  for (int f_id = 0 ; f_id < n_fields ; f_id++) {
    cs_field_t *f = cs_field_by_id(f_id);
    if (f->type & CS_FIELD_VARIABLE) {
      cs_field_get_key_struct(f, cs_field_key_id("solving_info"), &sinfo);
      cved = cved && (sinfo.l2residual  <= 1e-06);
    }
  }

  cved = cved && (ts->nt_cur-ts->nt_prev > 1) && (ts->nt_cur < ts->nt_max);

  if (cved) {
    ts->nt_max = ts->nt_cur +1;
    bft_printf("Converged at %d\n",ts->nt_max);
  }

  /* write header of residuals.dat */
  if (cs_glob_rank_id <= 0 && ts->nt_cur == 1) {
    file = fopen("residuals.dat", "w");
    fprintf(file,"#    Time step           Time");
    for (int f_id = 0 ; f_id < n_fields ; f_id++) {
      cs_field_t *f = cs_field_by_id(f_id);
      if (f->type & CS_FIELD_VARIABLE) fprintf(file,"           %s", f->name);
    }
    fprintf(file,"\n");
    fclose(file);
  }

  /* write residuals at log frequency */
  if (   cs_glob_rank_id <= 0
      && (   ts->nt_cur == 2
          || ts->nt_cur % cs_glob_log_frequency == 0
          || ts->nt_cur == ts->nt_max)) {
    file = fopen("residuals.dat", "a");
    fprintf(file, "     %14d        %14.5e",ts->nt_cur, ts->t_cur);
    for (int f_id = 0 ; f_id < n_fields ; f_id++) {
      cs_field_t *f = cs_field_by_id(f_id);
      if (f->type & CS_FIELD_VARIABLE) {
        cs_field_get_key_struct(f, cs_field_key_id("solving_info"), &sinfo);
        fprintf(file,"        %14.5e", sinfo.l2residual);
      }
    }
    fprintf(file,"\n");
    fclose(file);
  }

  /*===============================================================================
   * Compute 1D solution at last time step
   *===============================================================================*/

  if (ts->nt_cur == ts->nt_max) {
    /* nptx = number of points in x direction */
    /* nitmx = maximum iteration number for the Newton algorithm */

    nptx = 100;
    nitmx = 10000;

    cs_fluid_properties_t *fp = cs_get_glob_fluid_properties();
    cs_real_t *sec, *xsec, *pres, *mach, *rho, *temp, pe2, p2, pe1;
    cs_real_t *sound, *vel, gama, rg, pe, xyz[3], rho0, aire, machi;
    cs_real_t mache, smin, orig, slope, radius2, secstar, secstar1;
    cs_real_t secstar2, xmin, machtmp, rhotmp, temptmp, prestmp;
    cs_real_t soundtmp, p0left, t0left, rho0left, p0right, t0right;
    cs_real_t rho0right, secshock, machleft, machright, massflow;
    cs_real_t eps, mache1, mache2, machia, machib;
    int iswmax, isweep, iwarn;
    const cs_real_t pc_r = cs_physical_constants_r;

    BFT_MALLOC(sec, nptx, cs_real_t);
    BFT_MALLOC(xsec, nptx, cs_real_t);
    BFT_MALLOC(pres, nptx, cs_real_t);
    BFT_MALLOC(mach, nptx, cs_real_t);
    BFT_MALLOC(rho, nptx, cs_real_t);
    BFT_MALLOC(temp, nptx, cs_real_t);
    BFT_MALLOC(sound, nptx, cs_real_t);
    BFT_MALLOC(vel, nptx, cs_real_t);

    gama = 1. + pc_r/(fp->xmasmr*fp->cp0 - pc_r);
    rg = pc_r/fp->xmasmr;

    /* pe = static pressure at the exhaust (outlet) */

    /* Shock */
    if (testcase == 1)
      pe = 7.5e4;
    /* Subsonic */
    if (testcase == 2)
      pe = 9.5e4;
    /* Supersonic */
    if (testcase == 3)
      pe = 1.e4;

    rho0 = fp->p0/(rg*fp->t0);
    bft_printf("  ** COMPUTATION OF 1D SOLUTION OF A FLOW IN A LAVAL NOZZLE\n"
               "  ---------------------------------------------------------\n"
               "--------------------------------------------------------------\n"
               "  Upstream stagnation conditions of the nozzle\n"
               "  p0   (Pa)    : %14.5e\n"
               "  T0   (K)     : %14.5e\n"
               "  rho0 (kg/m3) : %14.5e\n"
               "--------------------------------------------------------------\n",
               fp->p0, fp->t0, rho0);

    /* Retrieve mesh nodes */
    for (cs_lnum_t ii = 0 ; ii < nptx ; ii++) {
      xyz[0] = (cs_real_t)ii/(cs_real_t)(nptx-1)*0.32;
      xyz[1] = 0.;
      xyz[2] = 0.;

      CS_PROCF(findpt, FINDPT)(&n_cells_ext, &n_cells, (cs_real_t *)cell_cen,
                               &xyz[0], &xyz[1], &xyz[2],
                               &iel,    &irangv);
      if (cs_glob_rank_id == irangv) {
        iel--;
        xsec[ii] = cell_cen[iel][0];
      }

      if (cs_glob_rank_id >= 0) {
        cs_parall_bcast(irangv,1,CS_DOUBLE,&xsec[ii]);
      }
    }

    if (cs_glob_rank_id <= 0) {
      xmin = 0.1446;
      for (cs_lnum_t ii = 0 ; ii < nptx ; ii++) {
        if (xsec[ii] <= 0.0265) {
          sec[ii] = 0.06;
        } else if (xsec[ii] > 0.0265 && xsec[ii] <= 0.0565) {
          radius2 = pow(0.06,2);
          sec[ii] = sqrt(radius2 - pow(xsec[ii]-0.0265,2));
        } else if (xsec[ii] > 0.0565 && xsec[ii] <= 0.0996) {
          slope = (0.027-0.0519)/(0.0996-0.0565);
          orig = 0.027-slope*0.0996;
          sec[ii] = slope*xsec[ii]+orig;
        } else if (xsec[ii] > 0.0996 && xsec[ii] <= 0.1525) {
          radius2 = pow(0.105-0.027,2)+pow(0.1446-0.0996,2);
          sec[ii] = 0.105-sqrt(radius2-pow(xsec[ii]-0.1446,2));
          smin = 0.105-sqrt(radius2-pow(xmin-0.1446,2));
        } else {
          slope = (0.03-0.0153)/(0.32-0.1525);
          orig = 0.0153-slope*0.1525;
          sec[ii] = slope*xsec[ii] + orig;
        }
      }
      for (cs_lnum_t ii = 0 ; ii < nptx ; ii++) {
        xsec[ii] -= xmin;
        sec[ii] = 2.*sec[ii];
      }
      smin = 2.*smin;

      /* Step 1 : Calculation of the two exhaust pressure separating the three regimes */

      /* Calculation of the exhaust Mach number thanks to the areas law */
      aire = sec[nptx-1]/smin;
      machi = 0.1;
      iwarn = 0.;
      newton(nitmx, iwarn, gama, machi, aire, &mache1);
      machi = 1.5;
      newton(nitmx, iwarn, gama, machi, aire, &mache2);

      /* Calculation of the pressure pe1 */
      pres_isent(gama, mache1, &pe1);
      pe1 = 1./pe1;
      pe1 *= fp->p0;

      /* Calculation of the pressure pe2 */
      pres_isent(gama, mache2, &p2);
      p2 = 1./p2;
      p2 *= fp->p0;
      pres_shock(gama, mache2, &pe2);
      pe2 *= p2;

      bft_printf("  Exhaust Mach number and pressure separating the three different\n"
                 "  regimes of the nozzle\n"
                 "  me1      : %14.5e\n"
                 "  pe1 (Pa) : %14.5e\n"
                 "  me2      : %14.5e\n"
                 "  pe2 (Pa) : %14.5e\n"
                 "--------------------------------------------------------------\n",
                 mache1, pe1, mache2, pe2);

      /* Step 2 : Calculation of the solution : diverging subsonic nozzle */
      if (pe >= pe1) {
        bft_printf("  With pe (Pa) : %14.5e\n"
                   "  the diverging nozzle is subsonic\n"
                   "--------------------------------------------------------------\n",pe);

        /* Calculation of the exhaust Mach number */
        pe = fp->p0/pe;
        invpres_isent(gama,pe,&mache);
        pe = fp->p0/pe;

        /* Calculation of the critical section */
        aeras_law(gama, mache, &aire, NULL);
        secstar = sec[nptx-1]/aire;
        bft_printf("  Critical section (m2) : %14.5e\n"
                   "--------------------------------------------------------------\n",secstar);

        /* Calculation of the solution for each x position */
        file = fopen("nozzle_profile.dat","a");
        fprintf(file,"# x(m) S(m2) Mach P(Pa) rho(kg/m3) T(K) c(m2/s) u(m/s)\n");
        iwarn = 0;
        machi = 0.1;
        for (cs_lnum_t ii = 0 ; ii < nptx ; ii++) {
          aire = sec[ii]/secstar;
          newton(nitmx,iwarn,gama,machi,aire,&machtmp);
          pres_isent(gama,machtmp,&prestmp);
          rho_isent(gama,machtmp,&rhotmp);
          temp_isent(gama, machtmp, &temptmp);
          sound_velocity(gama,fp->p0/prestmp,rho0/rhotmp,&soundtmp);
          mach[ii] = machtmp;
          pres[ii] = fp->p0/prestmp;
          rho[ii] = rho0/rhotmp;
          temp[ii] = fp->t0/temptmp;
          sound[ii] = soundtmp;
          vel[ii] = machtmp*soundtmp;
          fprintf(file,"%14.5e %14.5e %14.5e %14.5e %14.5e %14.5e %14.5e %14.5e\n",
                       xsec[ii]+xmin,sec[ii],mach[ii],pres[ii],rho[ii],temp[ii],
                       sound[ii],vel[ii]);
        }
        fclose(file);

        /* Calculation of the mass flow rate,
         * and writing of the inlet/exhaust conditions */
        massflow = rho[nptx-1]*mach[nptx-1]*sound[nptx-1]*sec[nptx-1];
        bft_printf("  Mass flow rate(kg/s) : %14.5e\n"
                   "--------------------------------------------------------------\n"
                   "  Inlet conditions\n"
                   "  ui   (m/s)   : %14.5e\n"
                   "  Mi           : %14.5e\n"
                   "  pi   (Pa)    : %14.5e\n"
                   "  rhoi (kg/m3) : %14.5e\n"
                   "  Ti   (K)     : %14.5e\n"
                   "--------------------------------------------------------------\n"
                   "  Exhaust conditions\n"
                   "  ue   (m/s)   : %14.5e\n"
                   "  Me           : %14.5e\n"
                   "  pe   (Pa)    : %14.5e\n"
                   "  rhoe (kg/m3) : %14.5e\n"
                   "  Te   (K)     : %14.5e\n"
                   "--------------------------------------------------------------\n"
                   "Solution profiles are given in nozzle_profile.dat in this order\n"
                   "# x(m) S(m2) Mach P(Pa) rho(kg/m3) T(K) c(m2/s) u(m/s)\n",
                   massflow, vel[0],mach[0],pres[0],rho[0],temp[0],
                   vel[nptx-1],mach[nptx-1],pres[nptx-1],rho[nptx-1],temp[nptx-1]);

        /* Step 3 : Calculation of the solution : diverging subsonic nozzle */
      } else if (pe < pe2) {
        bft_printf("  With pe (Pa) : %14.5e\n"
                   "  the diverging nozzle is supersonic\n"
                   "--------------------------------------------------------------\n",pe);
        secstar = smin;
        bft_printf("  Critical section (m2) : %14.5e (throat)\n"
                   "--------------------------------------------------------------\n",secstar);

        /* Calculation of the solution for each x position */
        file = fopen("nozzle_profile.dat","a");
        fprintf(file,"# x(m) S(m2) Mach P(Pa) rho(kg/m3) T(K) c(m2/s) u(m/s)\n");
        iwarn = 0;
        for (cs_lnum_t ii = 0 ; ii < nptx ; ii++) {
          if (xsec[ii] < 0.)
            machi = 0.1;
          else
            machi = 1.1;
          aire = sec[ii]/secstar;
          newton(nitmx,iwarn,gama,machi,aire,&machtmp);
          pres_isent(gama,machtmp,&prestmp);
          rho_isent(gama,machtmp,&rhotmp);
          temp_isent(gama, machtmp, &temptmp);
          sound_velocity(gama,fp->p0/prestmp,rho0/rhotmp,&soundtmp);
          mach[ii] = machtmp;
          pres[ii] = fp->p0/prestmp;
          rho[ii] = rho0/rhotmp;
          temp[ii] = fp->t0/temptmp;
          sound[ii] = soundtmp;
          vel[ii] = machtmp*soundtmp;
          fprintf(file,"%14.5e %14.5e %14.5e %14.5e %14.5e %14.5e %14.5e %14.5e\n",
                       xsec[ii]+xmin,sec[ii],mach[ii],pres[ii],rho[ii],temp[ii],
                       sound[ii],vel[ii]);
        }
        fclose(file);

        /* Calculation of the mass flow rate,
         * and writing of the inlet/exhaust conditions */
        massflow = rho[nptx-1]*mach[nptx-1]*sound[nptx-1]*sec[nptx-1];
        bft_printf("  Mass flow rate(kg/s) : %14.5e\n"
                   "--------------------------------------------------------------\n"
                   "  Inlet conditions\n"
                   "  ui   (m/s)   : %14.5e\n"
                   "  Mi           : %14.5e\n"
                   "  pi   (Pa)    : %14.5e\n"
                   "  rhoi (kg/m3) : %14.5e\n"
                   "  Ti   (K)     : %14.5e\n"
                   "--------------------------------------------------------------\n"
                   "  Exhaust conditions\n"
                   "  ue   (m/s)   : %14.5e\n"
                   "  Me           : %14.5e\n"
                   "  pe   (Pa)    : %14.5e\n"
                   "  rhoe (kg/m3) : %14.5e\n"
                   "  Te   (K)     : %14.5e\n"
                   "--------------------------------------------------------------\n"
                   "Solution profiles are given in nozzle_profile.dat in this order\n"
                   "# x(m) S(m2) Mach P(Pa) rho(kg/m3) T(K) c(m2/s) u(m/s)\n",
                   massflow, vel[0],mach[0],pres[0],rho[0],temp[0],
                   vel[nptx-1],mach[nptx-1],pres[nptx-1],rho[nptx-1],temp[nptx-1]);

        /* Step 4 : Calculation of the solution : shock in the diverging nozzle */
      } else {
        bft_printf("  With pe (Pa) : %14.5e\n"
                   "  there is a schock in the diverging noozle\n"
                   "--------------------------------------------------------------\n",pe);
        secstar1 = smin;
        p0left = fp->p0;
        t0left = fp->t0;
        rho0left = rho0;

        /* Iterative process to calculate the downstream stagnation pressure */
        prestmp = pe;
        p0right = 1e12;
        eps = 1.e-15;
        iswmax = 100;
        isweep = 1;
        iwarn = 0;
        machi = 0.1;

        while (isweep <= iswmax && fabs(p0right-prestmp) >= eps) {
          p0right = prestmp;
          aire = (sec[nptx-1]/secstar1)*(p0right/p0left);
          newton(nitmx,iwarn,gama,machi,aire,&machtmp);
          pres_isent(gama,machtmp,&prestmp);
          prestmp = prestmp*pe;
          if (iwarn == 1) {
            bft_printf("  Iteration = %d p0right (Pa) = %14.5e residu = %14.5e\n",
                       isweep, prestmp, fabs(p0right-prestmp));
          }
          isweep++;
        }
        bft_printf("\n  dowstream stagnation pressure (Pa) : %14.5e\n",prestmp);

        /* Calculation of the Mach number upstream and downstream the shock */
        p0right = prestmp;
        t0right = fp->t0;
        rho0right = p0right/(rg*t0right);
        iwarn = 0;
        machia = 1.;
        machib = 5.0;
        dichotomy(iwarn,gama,machia,machib,p0right/p0left,&machleft);
        bft_printf("  Mach number upstream the shock   : %14.5e\n",machleft);
        mach_shock(gama,machleft,&machright);
        bft_printf("  Mach number downstream the shock   : %14.5e\n",machright);
        bft_printf("--------------------------------------------------------------\n");

        /* Calculation of the shock section */
        secstar2 = (p0left/p0right)*secstar1;
        aeras_law(gama,machright,&secshock,NULL);
        secshock = secshock*secstar2;
        bft_printf("  Shock section (m2) : %14.5e such that:\n"
                   "  S(throat) = %14.5e < S(shock) = %14.5e < S(exhaust) = %14.5e\n",
                   secshock,smin,secshock,sec[nptx-1]);

        /* Calculation of the solution for each x position */
        file = fopen("nozzle_profile.dat","a");
        fprintf(file,"# x(m) S(m2) Mach P(Pa) rho(kg/m3) T(K) c(m2/s) u(m/s)\n");
        iwarn = 0;
        for (cs_lnum_t ii = 0 ; ii < nptx ; ii++) {
          if (xsec[ii] < 0) {
            machi = 0.1;
            aire = sec[ii]/secstar1;
            newton(nitmx,iwarn,gama,machi,aire,&machtmp);
            pres_isent(gama,machtmp,&prestmp);
            rho_isent(gama,machtmp,&rhotmp);
            temp_isent(gama,machtmp,&temptmp);
            sound_velocity(gama,p0left/prestmp,rho0left/rhotmp,&soundtmp);
            mach[ii] = machtmp;
            pres[ii] = p0left/prestmp;
            rho[ii] = rho0left/rhotmp;
            temp[ii] = t0left/temptmp;
            sound[ii] = soundtmp;
            vel[ii] = machtmp*soundtmp;
            fprintf(file,"%14.5e %14.5e %14.5e %14.5e %14.5e %14.5e %14.5e %14.5e\n",
                         xsec[ii]+xmin,sec[ii],mach[ii],pres[ii],rho[ii],temp[ii],
                         sound[ii],vel[ii]);
          } else if (xsec[ii]>= 0. && sec[ii] <= secshock) {
            machi = 1.1;
            aire = sec[ii]/secstar1;
            newton(nitmx,iwarn,gama,machi,aire,&machtmp);
            pres_isent(gama,machtmp,&prestmp);
            rho_isent(gama,machtmp,&rhotmp);
            temp_isent(gama,machtmp,&temptmp);
            sound_velocity(gama,p0left/prestmp,rho0left/rhotmp,&soundtmp);
            mach[ii] = machtmp;
            pres[ii] = p0left/prestmp;
            rho[ii] = rho0left/rhotmp;
            temp[ii] = t0left/temptmp;
            sound[ii] = soundtmp;
            vel[ii] = machtmp*soundtmp;
            fprintf(file,"%14.5e %14.5e %14.5e %14.5e %14.5e %14.5e %14.5e %14.5e\n",
                         xsec[ii]+xmin,sec[ii],mach[ii],pres[ii],rho[ii],temp[ii],
                         sound[ii],vel[ii]);
          } else {
            machi = 0.1;
            aire = sec[ii]/secstar2;
            newton(nitmx,iwarn,gama,machi,aire,&machtmp);
            pres_isent(gama,machtmp,&prestmp);
            rho_isent(gama,machtmp,&rhotmp);
            temp_isent(gama,machtmp,&temptmp);
            sound_velocity(gama,p0left/prestmp,rho0left/rhotmp,&soundtmp);
            mach[ii] = machtmp;
            pres[ii] = p0right/prestmp;
            rho[ii] = rho0right/rhotmp;
            temp[ii] = t0right/temptmp;
            sound[ii] = soundtmp;
            vel[ii] = machtmp*soundtmp;
            fprintf(file,"%14.5e %14.5e %14.5e %14.5e %14.5e %14.5e %14.5e %14.5e\n",
                         xsec[ii]+xmin,sec[ii],mach[ii],pres[ii],rho[ii],temp[ii],
                         sound[ii],vel[ii]);
          }
        }
        fclose(file);

        /* Calculation of the mass flow rate,
         * and writing of the inlet/exhaust conditions */
        massflow = rho[0]*mach[0]*sound[0]*sec[0];
        bft_printf("  Mass flow rate(kg/s) : %14.5e\n"
                   "--------------------------------------------------------------\n"
                   "  Inlet conditions\n"
                   "  ui   (m/s)   : %14.5e\n"
                   "  Mi           : %14.5e\n"
                   "  pi   (Pa)    : %14.5e\n"
                   "  rhoi (kg/m3) : %14.5e\n"
                   "  Ti   (K)     : %14.5e\n"
                   "--------------------------------------------------------------\n"
                   "  Exhaust conditions\n"
                   "  ue   (m/s)   : %14.5e\n"
                   "  Me           : %14.5e\n"
                   "  pe   (Pa)    : %14.5e\n"
                   "  rhoe (kg/m3) : %14.5e\n"
                   "  Te   (K)     : %14.5e\n"
                   "--------------------------------------------------------------\n"
                   "Solution profiles are given in nozzle_profile.dat in this order\n"
                   "# x(m) S(m2) Mach P(Pa) rho(kg/m3) T(K) c(m2/s) u(m/s)\n",
                   massflow, vel[0],mach[0],pres[0],rho[0],temp[0],
                   vel[nptx-1],mach[nptx-1],pres[nptx-1],rho[nptx-1],temp[nptx-1]);

      }
    }
    BFT_FREE(xsec);
    BFT_FREE(sec);
    BFT_FREE(mach);
    BFT_FREE(pres);
    BFT_FREE(rho);
    BFT_FREE(temp);
    BFT_FREE(sound);
    BFT_FREE(vel);
  }
}

END_C_DECLS
