/*============================================================================
 * Useful functions (perfect gas law)
 *============================================================================*/

/*----------------------------------------------------------------------------*/

/* VERS */

/*
  This file is part of Code_Saturne, a general-purpose CFD tool.

  Copyright (C) 1998-2016 EDF S.A.

  This program is free software; you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free Software
  Foundation; either version 2 of the License, or (at your option) any later
  version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
  details.

  You should have received a copy of the GNU General Public License along with
  this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
  Street, Fifth Floor, Boston, MA 02110-1301, USA.
*/

/*----------------------------------------------------------------------------*/

#include "cs_defs.h"

/*----------------------------------------------------------------------------
 * Standard C library headers
 *----------------------------------------------------------------------------*/

#include <assert.h>
#include <math.h>
#include <stdlib.h>

#if defined(HAVE_MPI)
#include <mpi.h>
#endif

/*----------------------------------------------------------------------------
 * PLE library headers
 *----------------------------------------------------------------------------*/

#include <ple_coupling.h>

/*----------------------------------------------------------------------------
 *  Local headers
 *----------------------------------------------------------------------------*/

#include "bft_mem.h"
#include "bft_error.h"
#include "bft_printf.h"

#include "fvm_writer.h"

#include "cs_base.h"
#include "cs_field.h"
#include "cs_field_pointer.h"
#include "cs_field_operator.h"
#include "cs_mesh.h"
#include "cs_mesh_quantities.h"
#include "cs_halo.h"
#include "cs_halo_perio.h"
#include "cs_log.h"
#include "cs_parameters.h"
#include "cs_prototypes.h"
#include "cs_rotation.h"
#include "cs_time_moment.h"
#include "cs_time_step.h"
#include "cs_turbomachinery.h"
#include "cs_selector.h"

#include "cs_post.h"

/*----------------------------------------------------------------------------
 *  Header for the current file
 *----------------------------------------------------------------------------*/

#include "1d_solution_functions.h"

/*----------------------------------------------------------------------------*/

void sound_velocity(cs_real_t gama,
                    cs_real_t pres,
                    cs_real_t rho,
                    cs_real_t *sound)
{
  *sound = sqrt(gama*pres/rho);
}

/*----------------------------------------------------------------------------*/

void aeras_law(cs_real_t gama,
               cs_real_t mach,
               cs_real_t *sec,
               cs_real_t *dsec)
{
  *sec = 1./mach*pow(2./(1.+gama)*(1.+0.5*(gama-1.)*pow(mach,2)),
                   (1.+gama)/(2.*(gama-1.)));
  if (dsec != NULL)
    *dsec = *sec/mach*(2.*(pow(mach,2)-1.)/(2.+(gama-1.)*pow(mach,2)));
}

/*----------------------------------------------------------------------------*/

void pres_isent(cs_real_t gama,
                cs_real_t mach,
                cs_real_t *pres)
{
  *pres = pow(1.+0.5*(gama-1.)*pow(mach,2),gama/(gama-1.));
}

/*----------------------------------------------------------------------------*/

void invpres_isent(cs_real_t gama,
                   cs_real_t pres,
                   cs_real_t *mach)
{
  *mach = pow(pres,(gama-1.)/gama)-1.;
  if (*mach >= 0.) {
    *mach = sqrt((2./(gama-1.))*(*mach));
  } else {
    bft_printf("Error : the isentropic pressure ratio should be greater than"
               "one to get the Mach number from it\n");
    cs_exit(EXIT_FAILURE);
  }
}

/*----------------------------------------------------------------------------*/

void rho_isent(cs_real_t gama,
               cs_real_t mach,
               cs_real_t *rho)
{
  *rho = pow(1.+0.5*(gama-1.)*pow(mach,2),1./(gama-1.));
}

/*----------------------------------------------------------------------------*/

void temp_isent(cs_real_t gama,
                cs_real_t mach,
                cs_real_t *temp)
{
  *temp = 1.+0.5*(gama-1.)*pow(mach,2);
}

/*----------------------------------------------------------------------------*/

void pres_shock(cs_real_t gama,
                cs_real_t mach,
                cs_real_t *pres)
{
  *pres = 1.+(2.*gama/(1.+gama))*(pow(mach,2)-1.);
}

/*----------------------------------------------------------------------------*/

void mach_shock(cs_real_t gama,
                cs_real_t mach1,
                cs_real_t *mach2)
{
  *mach2 = (1.+0.5*(gama-1)*pow(mach1,2))/(gama*pow(mach1,2)-0.5*(gama-1.));
  if (*mach2 >= 0.) {
    *mach2 = sqrt(*mach2);
  } else {
    bft_printf("Error : the function giving the Mach number after the shock"
               "is non positive\n");
    cs_exit(EXIT_FAILURE);
  }
}

/*----------------------------------------------------------------------------*/

void presgen_shock(cs_real_t gama,
                   cs_real_t mach,
                   cs_real_t *pres,
                   cs_real_t *dpres)
{
  *pres = pow(1.+((2.*gama)/(1.+gama))*(mach*mach-1.),-1./(gama-1.))
        *pow(1.-(2./(1.+gama))*(1.-1./(mach*mach)),-gama/(gama-1.));

  if (dpres != NULL)
    *dpres = -(4.*gama/(pow(gama,2)-1.))*mach*(*pres)
           * (1./(1.+2*gama/(1.+gama)*(pow(mach,2)-1.))
           + 1./(pow(mach,4)*(1.-(2./(1+gama))*(1.-1./pow(mach,2)))));
}

/*----------------------------------------------------------------------------*/

void newton(int       nitmx,
            int       iwarni,
            cs_real_t gama,
            cs_real_t mach0,
            cs_real_t sec0,
            cs_real_t *mach)
{
  cs_real_t eps, phi, dphi, f;

  /* Initialisation du newton */
  eps = 1.e-14;
  *mach = mach0;

  for (int inewt = 0 ; inewt < nitmx ; inewt++) {
    /* Declaration de la fonction et de sa derivee */
    aeras_law(gama,*mach,&f,&dphi);
    phi = f-sec0;

    /* Calcul du point suivant */
    *mach -= phi/dphi;
    if (fabs(phi) < eps) {
      if (iwarni == 1)
        bft_printf("Newton : it = %d  phi = %14.5f \n",inewt,phi);
      return;
    } else if (inewt == nitmx) {
      bft_printf(" --- Error: non convergence of the newton algorithm --- %d  %14.5f\n",
                 inewt, phi);
      cs_exit(EXIT_FAILURE);
    }

  if (iwarni == 1)
    bft_printf("Newton : it = %d  phi = %14.5f\n",inewt,phi);

  }
}

/*----------------------------------------------------------------------------*/

void newton2(int       nitmx,
             int       iwarni,
             cs_real_t gama,
             cs_real_t mach0,
             cs_real_t pres0,
             cs_real_t *mach)
{
  cs_real_t eps, phi, dphi, f;

  /* Initialisation du newton */
  eps = 1.e-15;
  *mach = mach0;

  for (int inewt = 0 ; inewt < nitmx ; inewt++) {
    /* Declaration de la fonction et de sa derivee */
    presgen_shock(gama,*mach,&f,&dphi);
    phi = f-pres0;

    /* Calcul du point suivant */
    *mach -= phi/dphi;
    if (fabs(phi) < eps) {
      if (iwarni == 1)
        bft_printf("Newton : it = %d  phi = %14.5f \n",inewt,phi);
      return;
    } else if (inewt == nitmx) {
      bft_printf(" --- Error: non convergence of the newton algorithm --- %d  %14.5f\n",
                 inewt, phi);
      cs_exit(EXIT_FAILURE);
    }

  if (iwarni == 1)
    bft_printf("Newton : it = %d  phi = %14.5f\n",inewt,phi);

  }
}

/*----------------------------------------------------------------------------*/

void dichotomy(int iwarni,
               cs_real_t gama,
               cs_real_t mach1,
               cs_real_t mach2,
               cs_real_t pres0,
               cs_real_t *mach)
{
  int isweep;
  cs_real_t presmid, presa, presb, macha, machb;
  cs_real_t machmid, prod, proda, prodb, eps;

  eps = 1.e-15;
  isweep = 1;
  presmid = 1.e12;
  macha = mach1;
  machb = mach2;

  presgen_shock(gama,macha,&presa,NULL);
  presgen_shock(gama,machb,&presb,NULL);

  prod = (presa-pres0)*(presb-pres0);

  if (prod >= 0.) {
    bft_printf("--- error: the initialisation of the dichotomy should be checked ---\n");
    cs_exit(EXIT_FAILURE);
  }

  while (fabs(presmid-pres0) > eps) {
    machmid = 0.5*(macha + machb);
    presgen_shock(gama, macha, &presa, NULL);
    presgen_shock(gama, machb, &presb, NULL);
    presgen_shock(gama, machmid, &presmid, NULL);
    proda = (presmid-pres0)*(presa-pres0);
    prodb = (presmid-pres0)*(presb-pres0);

    if (proda <= 0.) {
      macha = macha;
      machb = machmid;
    } else if (prodb <= 0.) {
      macha = machmid;
      machb = machb;
    }

    if (iwarni == 1) {
      bft_printf("Dichotomy, it = %d  residu = %14.5f\n",isweep,fabs(presmid-pres0));
    }
    isweep++;
  }

  if (iwarni == 1) {
    bft_printf("Dichotomy, it = %d  residu = %14.5f\n",isweep,fabs(presmid-pres0));
  }

  *mach = machmid;
}

/*----------------------------------------------------------------------------*/
